@@ -3,9 +3,11 @@
 'Inventory'
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
+from trytond.pyson import Not, Equal, Eval, Or, Bool
+from trytond.backend import TableHandler
 
 STATES = {
-    'readonly': "state != 'draft'",
+    'readonly': Not(Equal(Eval('state'), 'draft')),
 }
 
 
@@ -18,19 +20,22 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
     location = fields.Many2One(
         'stock.location', 'Location', required=True,
         domain=[('type', '=', 'storage')], states={
-            'readonly': "state != 'draft' or bool(lines)",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                Bool(Eval('lines'))),
         })
     date = fields.Date('Date', required=True, states={
-            'readonly': "state != 'draft' or bool(lines)",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                Bool(Eval('lines'))),
         })
     lost_found = fields.Many2One(
         'stock.location', 'Lost and Found', required=True,
         domain=[('type', '=', 'lost_found')], states=STATES)
     lines = fields.One2Many(
         'stock.inventory.line', 'inventory', 'Lines', states=STATES)
-    company = fields.Many2One(
-        'company.company', 'Company', required=True, states={
-            'readonly': "state != 'draft' or bool(lines)",
+    company = fields.Many2One('company.company', 'Company', required=True,
+        states={
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                Bool(Eval('lines'))),
         })
     state = fields.Selection([
         ('draft', 'Draft'),
@@ -42,6 +47,13 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
         super(Inventory, self).__init__()
         self._order.insert(0, ('date', 'DESC'))
 
+    def init(self, cursor, module_name):
+        super(Inventory, self).init(cursor, module_name)
+
+        # Add index on create_date
+        table = TableHandler(cursor, self, module_name)
+        table.index_action('create_date', action='add')
+
     def default_state(self, cursor, user, context=None):
         return 'draft'
 
@@ -50,7 +62,6 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
         return date_obj.today(cursor, user, context=context)
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -65,11 +76,6 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
             return location_ids[0]
         return False
 
-    def set_state_cancel(self, cursor, user, ids, context=None):
-        self.write(cursor, user, ids, {
-            'state': 'cancel',
-            }, context=context)
-
     def set_state_draft(self, cursor, user, inventory_id, context=None):
         self.write(cursor, user, inventory_id, {
             'state': 'draft',
@@ -200,13 +206,14 @@ class InventoryLine(ModelSQL, ModelView):
 
     product = fields.Many2One('product.product', 'Product', required=True,
             domain=[('type', '=', 'stockable')], on_change=['product'])
-    uom = fields.Function('get_uom', type='many2one', relation='product.uom',
-            string='UOM')
-    unit_digits = fields.Function('get_unit_digits', type='integer',
-            string='Unit Digits')
+    uom = fields.Function(fields.Many2One('product.uom', 'UOM'), 'get_uom')
+    unit_digits = fields.Function(fields.Integer('Unit Digits'),
+            'get_unit_digits')
     expected_quantity = fields.Float('Expected Quantity',
-            digits="(16, unit_digits)", readonly=True)
-    quantity = fields.Float('Quantity', digits="(16, unit_digits)")
+            digits=(16, Eval('unit_digits', 2)), readonly=True,
+            depends=['unit_digits'])
+    quantity = fields.Float('Quantity', digits=(16, Eval('unit_digits', 2)),
+            depends=['unit_digits'])
     move = fields.Many2One('stock.move', 'Move', readonly=True)
     inventory = fields.Many2One('stock.inventory', 'Inventory', required=True,
             ondelete='CASCADE')
@@ -224,7 +231,7 @@ class InventoryLine(ModelSQL, ModelView):
     def default_unit_digits(self, cursor, user, context=None):
         return 2
 
-    def on_change_product(self, cursor, user, ids, vals, context=None):
+    def on_change_product(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
         uom_obj = self.pool.get('product.uom')
         res = {}
@@ -237,13 +244,13 @@ class InventoryLine(ModelSQL, ModelView):
             res['unit_digits'] = product.default_uom.digits
         return res
 
-    def get_uom(self, cursor, user, ids, name, arg, context=None):
+    def get_uom(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             res[line.id] = line.product.default_uom.id
         return res
 
-    def get_unit_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_unit_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             res[line.id] = line.product.default_uom.digits
