@@ -1,12 +1,13 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Move"
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import ModelView, ModelSQL, fields, OPERATORS
 from trytond.backend import TableHandler
+from trytond.pyson import In, Eval, Not, In, Equal, If, Get, Bool
 from decimal import Decimal
 
 STATES = {
-    'readonly': "(state in ('cancel', 'done'))",
+    'readonly': In(Eval('state'), ['cancel', 'done']),
 }
 
 
@@ -20,23 +21,25 @@ class Move(ModelSQL, ModelView):
                 'from_location', 'to_location'],
             domain=[('type', '!=', 'service')])
     uom = fields.Many2One("product.uom", "Uom", required=True, states=STATES,
-            domain=["('category', '=', " \
-                    "(product, 'product.default_uom.category'))"],
-            context="{'category': (product, 'product.default_uom.category')}",
+            domain=[
+                ('category', '=',
+                    (Eval('product'), 'product.default_uom.category')),
+            ],
+            context={
+                'category': (Eval('product'), 'product.default_uom.category'),
+            },
             on_change=['product', 'currency', 'uom', 'company',
                 'from_location', 'to_location'])
-    unit_digits = fields.Function('get_unit_digits', type='integer',
-            string='Unit Digits', on_change_with=['uom'])
+    unit_digits = fields.Function(fields.Integer('Unit Digits',
+        on_change_with=['uom']), 'get_unit_digits')
     quantity = fields.Float("Quantity", required=True,
-            digits="(16, unit_digits)", states=STATES)
+            digits=(16, Eval('unit_digits', 2)), states=STATES)
     from_location = fields.Many2One("stock.location", "From Location", select=1,
             required=True, states=STATES,
-            domain=["('type', 'not in', " \
-                    "('warehouse', 'view'))"])
+            domain=[('type', 'not in', ('warehouse', 'view'))])
     to_location = fields.Many2One("stock.location", "To Location", select=1,
             required=True, states=STATES,
-            domain=["('type', 'not in', " \
-                    "('warehouse', 'view'))"])
+            domain=[('type', 'not in', ('warehouse', 'view'))])
     shipment_in = fields.Many2One('stock.shipment.in', 'Supplier Shipment',
             readonly=True, select=1, ondelete='CASCADE')
     shipment_out = fields.Many2One('stock.shipment.out', 'Customer Shipment',
@@ -59,25 +62,27 @@ class Move(ModelSQL, ModelView):
         ], 'State', select=1, readonly=True)
     company = fields.Many2One('company.company', 'Company', required=True,
             states={
-                'readonly': "state != 'draft'",
-            }, domain=["('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))"])
+                'readonly': Not(Equal(Eval('state'), 'draft')),
+            }, domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     unit_price = fields.Numeric('Unit Price', digits=(16, 4),
             states={
-                'invisible': "not unit_price_required",
-                'required': "unit_price_required",
-                'readonly': "state != 'draft'",
+                'invisible': Not(Bool(Eval('unit_price_required'))),
+                'required': Bool(Eval('unit_price_required')),
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
     cost_price = fields.Numeric('Cost Price', digits=(16, 4), readonly=True)
     currency = fields.Many2One('currency.currency', 'Currency',
             states={
-                'invisible': "not unit_price_required",
-                'required': "unit_price_required",
-                'readonly': "state != 'draft'",
+                'invisible': Not(Bool(Eval('unit_price_required'))),
+                'required': Not(Bool(Eval('unit_price_required'))),
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
-    unit_price_required = fields.Function('get_unit_price_required',
-            type='boolean', string='Unit Price Required',
-            on_change_with=['from_location', 'to_location'])
+    unit_price_required = fields.Function(fields.Boolean('Unit Price Required',
+        on_change_with=['from_location', 'to_location']),
+        'get_unit_price_required')
 
     def __init__(self):
         super(Move, self).__init__()
@@ -129,6 +134,9 @@ class Move(ModelSQL, ModelView):
         table  = TableHandler(cursor, self, module_name)
         table.drop_constraint('check_packing_in_out')
 
+        # Add index on create_date
+        table.index_action('create_date', action='add')
+
     def default_planned_date(self, cursor, user, context=None):
         if context and context.get('planned_date'):
             return context.get('planned_date')
@@ -205,7 +213,6 @@ class Move(ModelSQL, ModelView):
         return 'draft'
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -224,8 +231,7 @@ class Move(ModelSQL, ModelView):
             return company.currency.id
         return False
 
-    def on_change_with_unit_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_unit_digits(self, cursor, user, vals, context=None):
         uom_obj = self.pool.get('product.uom')
         if vals.get('uom'):
             uom = uom_obj.browse(cursor, user, vals['uom'],
@@ -233,13 +239,13 @@ class Move(ModelSQL, ModelView):
             return uom.digits
         return 2
 
-    def get_unit_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_unit_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
             res[move.id] = move.uom.digits
         return res
 
-    def on_change_product(self, cursor, user, ids, vals, context=None):
+    def on_change_product(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
         uom_obj = self.pool.get('product.uom')
         currency_obj = self.pool.get('currency.currency')
@@ -281,7 +287,7 @@ class Move(ModelSQL, ModelView):
                 res['unit_price'] = unit_price
         return res
 
-    def on_change_uom(self, cursor, user, ids, vals, context=None):
+    def on_change_uom(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
         uom_obj = self.pool.get('product.uom')
         currency_obj = self.pool.get('currency.currency')
@@ -329,10 +335,10 @@ class Move(ModelSQL, ModelView):
             'from_location': from_location,
             'to_location': to_location,
             }
-        return self.on_change_with_unit_price_required(cursor, user, [],
+        return self.on_change_with_unit_price_required(cursor, user,
                 vals, context=context)
 
-    def on_change_with_unit_price_required(self, cursor, user, ids, vals,
+    def on_change_with_unit_price_required(self, cursor, user, vals,
             context=None):
         location_obj = self.pool.get('stock.location')
         if vals.get('from_location'):
@@ -347,8 +353,7 @@ class Move(ModelSQL, ModelView):
                 return True
         return False
 
-    def get_unit_price_required(self, cursor, user, ids, name, arg,
-            context=None):
+    def get_unit_price_required(self, cursor, user, ids, name, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
             res[move.id] = False
@@ -364,7 +369,7 @@ class Move(ModelSQL, ModelView):
                 return False
         return True
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -373,13 +378,8 @@ class Move(ModelSQL, ModelView):
             res[m.id] = "%s%s %s" % (m.quantity, m.uom.symbol, m.product.rec_name)
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            args2.append(('product', args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        return [('product',) + clause[1:]]
 
     def search(self, cursor, user, args, offset=0, limit=None, order=None,
             context=None, count=False, query_string=False):
@@ -389,10 +389,11 @@ class Move(ModelSQL, ModelView):
         def process_args(args):
             i = 0
             while i < len(args):
-                if isinstance(args[i], list):
-                    process_args(args[i])
-                if isinstance(args[i], tuple) \
-                        and args[i][0] == 'to_location_warehouse':
+                #add test for xmlrpc that doesn't handle tuple
+                if (isinstance(args[i], tuple) or \
+                        (isinstance(args[i], list) and len(args[i]) > 2 and \
+                        args[i][1] in OPERATORS)) and \
+                        args[i][0] == 'to_location_warehouse':
                     location_id = False
                     if args[i][2]:
                         location = location_obj.browse(cursor, user,
@@ -400,6 +401,8 @@ class Move(ModelSQL, ModelView):
                         if location.type == 'warehouse':
                             location_id = location.input_location.id
                     args[i] = ('to_location', args[i][1], location_id)
+                elif isinstance(args[i], list):
+                    process_args(args[i])
                 i += 1
         process_args(args)
         return super(Move, self).search(cursor, user, args, offset=offset,
@@ -424,6 +427,7 @@ class Move(ModelSQL, ModelView):
         """
         uom_obj = self.pool.get('product.uom')
         product_obj = self.pool.get('product.product')
+        product_template_obj = self.pool.get('product.template')
         location_obj = self.pool.get('stock.location')
         currency_obj = self.pool.get('currency.currency')
         company_obj = self.pool.get('company.company')
@@ -461,6 +465,13 @@ class Move(ModelSQL, ModelView):
         else:
             new_cost_price = product.cost_price
 
+        if hasattr(product_obj, 'cost_price'):
+            digits = product_obj.cost_price.digits
+        else:
+            digits = product_template_obj.cost_price.digits
+        new_cost_price = new_cost_price.quantize(
+                Decimal(str(10.0**-digits[1])))
+
         ctx = context.copy()
         ctx['user'] = user
         product_obj.write(
@@ -635,8 +646,7 @@ class Move(ModelSQL, ModelView):
         if context is None:
             context = {}
 
-        if cursor.has_lock():
-            cursor.execute('LOCK TABLE stock_move')
+        cursor.lock(self._table)
 
         local_ctx = context and context.copy() or {}
         local_ctx['stock_date_end'] = date_obj.today(cursor, user,
